from django.conf import settings

from confapp import conf
from pyforms_web.basewidget import BaseWidget


class FinancesDashboardWidget(BaseWidget):
    """
    """

    UID = 'finances'
    TITLE = 'Finances'

    # ORQUESTRA CONFIGURATION
    # =========================================================================
    ORQUESTRA_MENU = 'left'
    ORQUESTRA_MENU_ICON = 'dollar'
    ORQUESTRA_MENU_ORDER = 30
    LAYOUT_POSITION = conf.ORQUESTRA_HOME
    # =========================================================================

    AUTHORIZED_GROUPS = ['superuser', settings.PROFILE_LAB_ADMIN]



from django.conf import settings

from confapp import conf
from pyforms.controls import ControlCheckBox
from pyforms_web.web.middleware import PyFormsMiddleware
from pyforms_web.widgets.django import ModelAdminWidget

from finance.models import Project

from .financeproject_form import FinanceProjectFormApp


class FinanceProjectListApp(ModelAdminWidget):

    TITLE = 'Finance projects'

    MODEL = Project

    AUTHORIZED_GROUPS = ['superuser', settings.PROFILE_LAB_ADMIN]

    LIST_DISPLAY = [
        'code',
        'name',
        'start_date',
        'end_date',
    ]

    ADDFORM_CLASS = FinanceProjectFormApp
    EDITFORM_CLASS = FinanceProjectFormApp
    USE_DETAILS_TO_EDIT = False

    # ORQUESTRA CONFIGURATION
    # =========================================================================
    LAYOUT_POSITION = conf.ORQUESTRA_HOME
    # =========================================================================

    def __init__(self, *args, **kwargs):

        self._active = ControlCheckBox(
            'Active',
            default=True,
            label_visible=False,
            changed_event=self.populate_list,
            field_style='text-align:right;',
        )

        super().__init__(*args, **kwargs)

    def get_toolbar_buttons(self, has_add_permission=False):
        return tuple(
            (['_add_btn'] if has_add_permission else []) + [
                '_active',
            ]
        )

    def get_queryset(self, request, qs):

        if self._active.value:
            qs = qs.active()

        return qs

    def has_update_permissions(self, obj):
        if obj and obj.code == 'NO TRACK':
            return False
        else:
            return True

    def has_remove_permissions(self, obj):
        """Only superusers may delete these objects."""
        user = PyFormsMiddleware.user()
        return user.is_superuser



from django.conf import settings

from confapp import conf
from pyforms.basewidget import segment
from pyforms.controls import ControlCheckBox
from pyforms_web.web.middleware import PyFormsMiddleware
from pyforms_web.widgets.django import ModelAdminWidget

from finance.models import CostCenter

from .financeproject_list import FinanceProjectListApp


class CostCenterListApp(ModelAdminWidget):

    TITLE = 'Cost Centers'
    UID = 'cost-centers'

    MODEL = CostCenter

    LIST_DISPLAY = ['code', 'name', 'start_date', 'end_date']

    LIST_FILTER = ['group', 'project__code']

    SEARCH_FIELDS = ['name__icontains', 'code__icontains']

    INLINES = [FinanceProjectListApp]

    FIELDSETS = [
        segment(
            'name',
            ('code', 'start_date', 'end_date'),
            'group',
        ),
        ' ',
        'FinanceProjectListApp',
        ' ',
    ]

    # ORQUESTRA CONFIGURATION
    # =========================================================================
    ORQUESTRA_MENU = 'left>FinancesDashboardWidget'
    ORQUESTRA_MENU_ORDER = 10
    ORQUESTRA_MENU_ICON = 'boxes'
    LAYOUT_POSITION = conf.ORQUESTRA_HOME
    # =========================================================================

    AUTHORIZED_GROUPS = ['superuser', settings.PROFILE_LAB_ADMIN]

    def __init__(self, *args, **kwargs):

        self._active = ControlCheckBox(
            'Active',
            default=True,
            label_visible=False,
            changed_event=self.populate_list,
            # field_style='text-align:right;',  # FIXME breaks form
        )

        super().__init__(*args, **kwargs)

        # Edit filter label
        self._list.custom_filter_labels = {
            'project__code': 'Project Code',
        }

    def get_toolbar_buttons(self, has_add_permission=False):
        return tuple(
            (['_add_btn'] if has_add_permission else []) + [
                '_active',
            ]
        )

    def get_queryset(self, request, qs):

        if self._active.value:
            qs = qs.active()

        return qs

    def has_remove_permissions(self, obj):
        """Only superusers may delete these objects."""
        user = PyFormsMiddleware.user()
        return user.is_superuser



from .budget import BudgetAdminWidget
from .dashboard import FinancesDashboardWidget
from .costcenter.costcenter_list import CostCenterListApp
from .costcenter.expensecode_list import ExpenseCodeListApp
from .costcenter.financeproject_list import FinanceProjectListApp
from .grant_domain_app import GrantDomainAdminApp

from .financing_agency_app import FinancingAgencyAdminApp

from .grant_app import GrantsAdminApp




from django.db import models

class ExpenseCode(models.Model):
    """
    Represents a Budget of an order in the system
    """

    number = models.CharField('ExpenseCode Number', max_length=100) #: Name
    type = models.CharField('ExpenseCode Type', max_length=100) #: Type

    project = models.ForeignKey('finance.Project', verbose_name='Finance project', on_delete=models.CASCADE)

    @staticmethod
    def autocomplete_search_fields():
        return (
            "number__icontains",
            'type__icontains',
            'project__name__icontains',
            'project__code__icontains',
            'project__costcenter__name__icontains'
        )


    class Meta:
        verbose_name = "Expense code"
        verbose_name_plural = "Expense codes"
        unique_together = ("number", "project")

    def __str__(self):
        return self.project.costcenter.code + "-" + \
               self.project.code + "-" + \
               self.number + ": " + self.project.costcenter.name + "-" + \
               self.project.name + "-" + self.type

    def expensecode(self):
        return str(self.project.costcenter.code + "-" + \
                   self.project.code + "-" + \
                   self.number + ": " + self.project.costcenter.name + "-" + \
                   self.project.name + "-" + self.type)

    @property
    def abbrv(self):
        """Returns a string with codes only, i.e. `CCCCCCC-PPP-EE`."""
        cost_center = self.project.costcenter.code
        project = self.project.code
        expense_code = self.number
        return f'{cost_center}-{project}-{expense_code}'



# Generated by Django 2.2.4 on 2019-08-06 15:18

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('people', '0003_auto_20190806_1516'),
        ('common', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='BudgetDoc',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('document', models.FileField(upload_to='documents/%Y/%m/%d')),
                ('year', models.IntegerField(verbose_name='Year')),
            ],
            options={
                'verbose_name': 'Budget document',
            },
        ),
        migrations.CreateModel(
            name='CostCenter',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200, verbose_name='Name')),
                ('code', models.CharField(max_length=200, verbose_name='Code')),
                ('start_date', models.DateField(blank=True, null=True)),
                ('end_date', models.DateField(blank=True, null=True)),
                ('group', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='people.Group', verbose_name='Group')),
            ],
            options={
                'verbose_name': 'Finance Cost Center',
                'verbose_name_plural': 'Finance Cost Centers',
                'ordering': ['code'],
            },
        ),
        migrations.CreateModel(
            name='FinancingAgency',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200, verbose_name='Name')),
            ],
            options={
                'verbose_name': 'Financing agency',
                'verbose_name_plural': 'Financing agencies',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='Grant',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200, verbose_name='Name')),
            ],
            options={
                'verbose_name': 'Grant',
                'verbose_name_plural': 'Grants',
            },
        ),
        migrations.CreateModel(
            name='Grantdomain',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('domain', models.CharField(max_length=200, verbose_name='Grant domain')),
            ],
            options={
                'verbose_name': 'Grant domain',
                'verbose_name_plural': 'Grant domains',
            },
        ),
        migrations.CreateModel(
            name='Project',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200, verbose_name='Name')),
                ('code', models.CharField(max_length=200, unique=True, verbose_name='Code')),
                ('start_date', models.DateField(blank=True, help_text='Starting date of the grant', null=True, verbose_name='Starting date')),
                ('end_date', models.DateField(blank=True, help_text='Ending date of the grant', null=True, verbose_name='Ending date')),
                ('total_amount', models.DecimalField(blank=True, decimal_places=2, help_text='Total amount of the grant', max_digits=11, null=True, verbose_name='Total amount')),
                ('overheads', models.DecimalField(blank=True, decimal_places=2, help_text='Overhead amount of the grant', max_digits=11, null=True, verbose_name='Overhead')),
                ('funding', models.DecimalField(blank=True, decimal_places=2, help_text='Funding amount of the grant', max_digits=11, null=True, verbose_name='Funding')),
                ('costcenter', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='finance.CostCenter', verbose_name='Cost Center')),
                ('currency', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='common.Currency', verbose_name='Currency')),
                ('grant', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='finance.Grant', verbose_name='Grant')),
                ('responsible', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='people.Person')),
            ],
            options={
                'verbose_name': 'Finance Project',
                'verbose_name_plural': 'Finance Projects',
                'ordering': ['code', 'costcenter'],
            },
        ),
        migrations.AddField(
            model_name='grant',
            name='domain',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='finance.Grantdomain', verbose_name='Grant domain'),
        ),
        migrations.AddField(
            model_name='grant',
            name='grantor',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='finance.FinancingAgency', verbose_name='Grantor'),
        ),
        migrations.AddField(
            model_name='grant',
            name='nationality',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='common.Nationality', verbose_name='Grant nationality'),
        ),
        migrations.CreateModel(
            name='ExpenseCode',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('number', models.CharField(max_length=100, verbose_name='ExpenseCode Number')),
                ('type', models.CharField(max_length=100, verbose_name='ExpenseCode Type')),
                ('project', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='finance.Project', verbose_name='Finance project')),
            ],
            options={
                'verbose_name': 'Expense code',
                'verbose_name_plural': 'Expense codes',
                'unique_together': {('number', 'project')},
            },
        ),
        migrations.CreateModel(
            name='Budget',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('amount', models.DecimalField(decimal_places=2, max_digits=11, verbose_name='Amount (NET)')),
                ('year', models.IntegerField(blank=True, null=True, verbose_name='Year')),
                ('expensecode', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='finance.ExpenseCode', verbose_name='Expense Code')),
            ],
            options={
                'verbose_name': 'Budget',
                'verbose_name_plural': 'Budgets',
                'unique_together': {('year', 'expensecode')},
            },
        ),
    ]



from django.db import models

class Grant(models.Model):
    """
    Represents a Grant in the system
    Example: PZLOC
    """
    name = models.CharField('Name', max_length=200)  #: Name

    nationality = models.ForeignKey('common.Nationality', verbose_name='Grant nationality', on_delete=models.CASCADE)
    domain      = models.ForeignKey('GrantDomain', verbose_name='Grant domain', on_delete=models.CASCADE)
    grantor     = models.ForeignKey('FinancingAgency', verbose_name='Grantor', on_delete=models.CASCADE)

    class Meta:
        verbose_name = "Grant"
        verbose_name_plural = "Grants"

    def __str__(self):
        return self.name



from confapp import conf
from pyforms.basewidget import segment
from pyforms_web.widgets.django import ModelFormWidget

from finance.models import Project

from .expensecode_list import ExpenseCodeListApp


class FinanceProjectFormApp(ModelFormWidget):

    TITLE = 'Finance project'

    MODEL = Project

    INLINES = [ExpenseCodeListApp]

    FIELDSETS = [
        ('grant', 'currency'),
        segment(
            ('name', 'code', 'responsible'),
            ('start_date', 'end_date', ' '),
            ('total_amount', 'importoverheads', 'funding'),
        ),
        ' ',
        'ExpenseCodeListApp',
        ' ',
    ]

    # ORQUESTRA CONFIGURATION
    # =========================================================================
    LAYOUT_POSITION = conf.ORQUESTRA_NEW_TAB
    # =========================================================================

    @property
    def title(self):
        obj = self.model_object
        if obj is None:
            return ModelFormWidget.title.fget(self)
        else:
            return "Proj: {0}".format(obj.name)

    @title.setter
    def title(self, value):
        ModelFormWidget.title.fset(self, value)



from django.conf import settings

from confapp import conf
from pyforms.basewidget import segment
from pyforms_web.web.middleware import PyFormsMiddleware
from pyforms_web.widgets.django import ModelAdminWidget

from finance.models import ExpenseCode


class ExpenseCodeListApp(ModelAdminWidget):

    TITLE = 'Expense codes'

    MODEL = ExpenseCode

    LIST_DISPLAY = ['number', 'type']

    FIELDSETS = [
        segment(
            # 'financeproject',  # not required if app used as Inline
            ('number', 'type')
        )
    ]

    # ORQUESTRA CONFIGURATION
    # =========================================================================
    LAYOUT_POSITION = conf.ORQUESTRA_HOME
    # =========================================================================

    AUTHORIZED_GROUPS = ['superuser', settings.PROFILE_LAB_ADMIN]

    def has_add_permissions(self):
        finance_project = self.parent_model.objects.get(pk=self.parent_pk)
        if finance_project.code == 'NO TRACK':
            return False
        else:
            return True

    def has_update_permissions(self, obj):
        if obj and obj.project.code == 'NO TRACK':
            return False
        else:
            return True

    def has_remove_permissions(self, obj):
        """Only superusers may delete these objects."""
        user = PyFormsMiddleware.user()
        return user.is_superuser
